package com.yspay.demo.transfer;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import hikefa.core.util.Sysdate;

import java.util.Date;

/**
 * 代付服务
 */

public class ActPayService {

    /**
     * 模板详情接口调用的公共参数初始化
     */
    public static JSONObject initfeeRuleDetailParam(String feeTemplateNo, String userName) {
        JSONObject map = new JSONObject();
        map.put("tradeSource", "4");
        map.put("charset", "UTF-8");
        map.put("timestamp", Sysdate.getTimeStr());
        map.put("feeTemplateNo", feeTemplateNo);
        map.put("priority", "1"); // 类型：String 必有字段
        // 备注：优先级(整数，且范围为[0,9999]),优先级越小越优先
        map.put("discount", 1); // 类型：Number 必有字段
        // 备注：收费折扣(1代表无折扣，0.8代表8折)，范围[0.01,1.00],仅支持小数点后2位
        map.put("initiator", userName);// 类型：String 必有字段 备注：发起人
        map.put("isLocal", "00");// 类型：String 必有字段 备注：是否是同城（00:不限;01:是;02:不是）
        return map;
    }


    /**
     * 初始化开通代付需要定义的模板详情
     * @param templeteNo 模板编号
     * @param userName 申请人
     * @param feeType 收费方式1:固定收费;2:按费率设置;
     * @param pubFee 对公费率
     * @param priFee 对私费率
     */
    public static JSONArray initActPayFeeRuleDetail(String templeteNo, String userName, String feeType, String pubFee, String priFee){
        JSONArray feeParamList = new JSONArray();
        JSONObject pubParam1 = initfeeRuleDetailParam(templeteNo,userName);
        pubParam1.put("paymode","06");
        pubParam1.put("feeType",feeType);
        pubParam1.put("fee",pubFee);
        pubParam1.put("accountType","21");//对公借记卡
        feeParamList.add(pubParam1);

        JSONObject pubParam2 = JSONObject.parseObject(JSON.toJSONString(pubParam1, SerializerFeature.DisableCircularReferenceDetect));
        pubParam2.put("accountType","23");//对公存折
        feeParamList.add(pubParam2);

        JSONObject pubParam3 = JSONObject.parseObject(JSON.toJSONString(pubParam1, SerializerFeature.DisableCircularReferenceDetect));
        pubParam3.put("accountType","24");//单位结算卡
        feeParamList.add(pubParam3);

        JSONObject priParam1 = initfeeRuleDetailParam(templeteNo,userName);
        priParam1.put("paymode","06");
        priParam1.put("feeType",feeType);
        priParam1.put("fee",priFee);
        priParam1.put("accountType","11");//对私借记卡
        feeParamList.add(priParam1);

        JSONObject priParam2 = JSONObject.parseObject(JSON.toJSONString(priParam1, SerializerFeature.DisableCircularReferenceDetect));
        priParam2.put("accountType","12");//对私贷记卡
        feeParamList.add(priParam2);

        JSONObject priParam3 = JSONObject.parseObject(JSON.toJSONString(priParam1, SerializerFeature.DisableCircularReferenceDetect));
        priParam3.put("accountType","13");//对私存折
        feeParamList.add(priParam3);
        return feeParamList;
    }


    /**
     * 企业代收付入网协议新增参数封装
     */
    public static JSONObject utilOnlineActPayMerc(String mercId, TbsOnlineUserFlumeLimit limit) {
        JSONObject param = new JSONObject();
        String timestamp = BcsUtils.getTimestampStr(new Date());
        param.put("tradeSource", "44"); // 类型：String 必有字段 备注：交易来源
        param.put("reqMsgId", timestamp); // 类型：String 必有字段 备注：请求流水号
        param.put("charset", "UTF-8"); // 类型：String 必有字段 备注：字符集
        param.put("timestamp", timestamp); // 类型：String
        param.put("mercId", mercId); // 类型：String 必有字段 备注：商户号
        param.put("serviceType","2"); //类型：String  必有字段  备注：协议类型，1-代收；2-代付
        param.put("protocolState","00"); //类型：String  必有字段  备注：协议状态，00-未生效、01-生效、02-失效
        param.put("dayPennumber",limit.getDayPennumber());               //类型：Number  必有字段  备注：日笔数
        param.put("dayAmount",limit.getDayAmt());               //类型：Number  必有字段  备注：日金额
        param.put("monthPennumber",limit.getMonthPennumbr());                //类型：Number  必有字段  备注：月笔数
        param.put("monthAmount",limit.getMonthAmt());                //类型：Number  必有字段  备注：月金额
        param.put("lowderlimitAmount",limit.getEachAmtMin());                //类型：Number  必有字段  备注：单笔最小金额
        param.put("upperlimitAmount",limit.getEachAmt()) ;               //类型：Number  必有字段  备注：单笔最大金额
        return param;
    }



}
