package com.yspay.demo.transfer;

import org.apache.commons.lang.StringUtils;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Random;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * 云商服的公共方法工具类
 */
public class BcsUtils {


    private final static String numeric="^\\d+(\\.\\d+)?$";

    /**
     * 用来判断文件字节流的编码是否是64位
     * @param base64Str 文件字节流Base64加密后的字符串
     */
    public static boolean isBase64Str(String base64Str) {
        String base64Pattern = "^([A-Za-z0-9+/]{4})*([A-Za-z0-9+/]{4}|[A-Za-z0-9+/]{3}=|[A-Za-z0-9+/]{2}==)$";
        return Pattern.matches(base64Pattern, base64Str);
    }


    public static boolean isNotBase64Str(String base64Str) {
        return !isBase64Str(base64Str);
    }

    /**
     * 获取时间戳流水
     */
    public static String getTimestampStr(Date date){
        if(date == null){
            return "";
        }
        SimpleDateFormat fmt = new SimpleDateFormat("yyyyMMddHHmmss");
        return fmt.format(date);
    }

    /**
     * 数字字符串小数点右移2位
     * @param numberStr 数字格式的字符串
     * @return 数字格式的字符串
     */
    public static String moveStrRight2Point(String numberStr){
        if (StringUtils.isBlank(numberStr)) {
            return null;
        }
        BigDecimal b = new BigDecimal(numberStr);
        return b.movePointRight(2).toString();
    }

    /**
     * 比较2个数值格式字符串的大小
     * @param numberStr1 值格式字符串1
     * @param numberStr2 值格式字符串2
     * @return str1>str2 返回1,str1<str2 返回-1 str1=str2 返回0
     */
    public static int compareNumberStr(String numberStr1,String numberStr2){
        BigDecimal val1 = new BigDecimal(numberStr1);
        BigDecimal val2 = new BigDecimal(numberStr2);
        return val1.compareTo(val2);
    }

    /**
     * 判断字符串是正数：包含正整数和正小数
     */
    public static boolean isNumeric(String str) {
        Pattern pattern = Pattern.compile(numeric);
        Matcher match = pattern.matcher(str);
        if (match.matches() == false) {
            return false;
        } else {
            return true;
        }
    }

    public static Double stringToDuble(String str){
        try {
            if (StringUtils.isNotBlank(str)) {
                return Double.valueOf(str);
            }
        } catch (Exception e) {
            return null;
        }
        return null;
    }


    public static String firstUpper(String str) {
        if(StringUtils.isBlank(str)){
            return null;
        }
        return str.substring(0, 1).toUpperCase() + str.substring(1);
    }
}
