package com.yspay.demo.transfer;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import hikefa.core.util.Sysdate;
import org.apache.commons.lang.StringUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;

/**
 * 扫码线上费率更新服务，不依赖云商服的业务表
 */

public class ScanOnlineFeeService {


    /**
     * 绑定模板审核参数封装
     */
    public static Map<String, Object> utilBandingAudit(String userName, String custId) {
        Map<String, Object> map = new HashMap<String, Object>();
        String timeStamp = BcsUtils.getTimestampStr(new Date());
        map.put("tradeSource", "44"); // 类型：String 必有字段 备注：交易来源
        map.put("reqMsgId",timeStamp); // 类型：String 必有字段 备注：请求流水号
        // 8-64位
        map.put("charset", "UTF-8"); // 类型：String 必有字段 备注：字符集
        map.put("timestamp", timeStamp); // 类型：String
        map.put("trantype", "311"); // 类型：String 必有字段 备注：商户号
        map.put("feeObject", "1"); // 类型：String 必有字段 备注：计费对象（0:发起方;1:收款方;2:付款方）
        map.put("custid", custId); // 类型：String 必有字段 备注：商户号
        map.put("auditor", userName + "操作"); // 类型：String 必有字段
        // 备注：计费方式（01:实时扣收;02:日结;03:月结）
        map.put("auditResult", "00"); // 类型：String 必有字段 备注：商户号
        return map;
    }

    public static JSONObject initUpdateFeeParam(String mercId){
        JSONObject updateFeeObje = new JSONObject();
        String timeStamp = BcsUtils.getTimestampStr(new Date());
        String feeTemplateNo = "TMS" + timeStamp;
        updateFeeObje.put("tradeSource", "44"); // 类型：String 必有字段 备注：交易来源
        updateFeeObje.put("reqMsgId", timeStamp); // 类型：String 必有字段 备注：请求流水号
        // 8-64位
        updateFeeObje.put("charset", "UTF-8"); // 类型：String 必有字段 备注：字符集
        updateFeeObje.put("timestamp",timeStamp); // 类型：String
        // 必有字段
        // 备注：时间戳
        // 格式yyyyMMddHHmmss
        updateFeeObje.put("feeTemplateNo",feeTemplateNo ); // 类型：String 必有字段

        updateFeeObje.put("trantype", "311"); // 即时到账
        updateFeeObje.put("feeTemplateName", mercId + "即时到账"); // 类型：String
        updateFeeObje.put("feeObject", "1");  //类型：String  必有字段  备注：计费对象（0:发起方;1:收款方;2:付款方）
        updateFeeObje.put("feeMode", "01"); // 类型：String 必有字段 备注：计费方式（01:实时扣收;02:日结;03:月结）
        updateFeeObje.put("custid", mercId); // 类型：String 必有字段 备注：商户号
        updateFeeObje.put("initiator", mercId); // 类型：String 必有字段 备注：商户号
        return updateFeeObje;
    }


    /**
     * 微信费率修改参数封装
     */
    public static JSONArray initWxUpdateParam(WxAliFeeVO wxAliFee){
        JSONArray updateScanWxObjList = new JSONArray();
        Map<String,Object> updateScanWxObj = initTempDetail();

        //微信反扫开通按百分比
        utilSegmentFee(updateScanWxObj, wxAliFee.getWechatSignRate(), wxAliFee.getWechatSignMin(), "999999.99");
        updateScanWxObj.put("banktype", "1902000");  //微信二维码
        updateScanWxObj.put("paymode", "23");
        updateScanWxObjList.add(updateScanWxObj);

        //开通小程序
        JSONObject wxMini1 = JSONObject.parseObject(JSON.toJSONString(updateScanWxObj, SerializerFeature.DisableCircularReferenceDetect));
        wxMini1.put("paymode", "29");
        wxMini1.remove("banktype");
        updateScanWxObjList.add(wxMini1);

        //开通公众号支付
        //SerializerFeature.DisableCircularReferenceDetect关闭引用检测后,$ref的问题就会被解决
        JSONObject wxMini2 = JSONObject.parseObject(JSON.toJSONString(updateScanWxObj, SerializerFeature.DisableCircularReferenceDetect));
        wxMini2.put("paymode", "28");
        wxMini2.remove("banktype");
        updateScanWxObjList.add(wxMini2);

        return updateScanWxObjList;
    }


    /**
     * 支付宝费率修改参数封装
     */
    public static JSONArray initZfbUpdateParam(WxAliFeeVO wxAliFee) {
        JSONArray updateScanZfbObjList = new JSONArray();
        JSONObject updateScanZfbObj = initTempDetail();

        //支付宝开通反扫业务按百分比
        utilSegmentFee(updateScanZfbObj, wxAliFee.getAlipaySignRate(), wxAliFee.getAlipaySignMin(), "999999.99");
        updateScanZfbObj.put("banktype", "1903000");  //微信二维码
        updateScanZfbObj.put("paymode", "23");
        updateScanZfbObjList.add(updateScanZfbObj);


        //开通支付宝生活支付
        JSONObject aliFront1 = JSONObject.parseObject(JSON.toJSONString(updateScanZfbObj, SerializerFeature.DisableCircularReferenceDetect));
        updateScanZfbObj.put("paymode", "26");
        updateScanZfbObjList.add(aliFront1);

        //支付宝开通扫描支付
        JSONObject aliFront2 = JSONObject.parseObject(JSON.toJSONString(updateScanZfbObj, SerializerFeature.DisableCircularReferenceDetect));
        aliFront2.put("paymode", "07");
        aliFront2.put("banktype", "1903000");
        updateScanZfbObjList.add(aliFront2);

        return updateScanZfbObjList;
    }


    public static JSONArray initBankUpdateParam(BankFeeVO bankFee1, BankFeeVO bankFee2) {
        JSONArray updateScanBankObjList = new JSONArray();

        //银联借贷记卡开通线上反扫费率模板
        JSONObject payUnionObject1 = initTempDetail(); // 借记卡
        payUnionObject1.put("bankAccountType", "11");
        payUnionObject1.put("feeType", "3");
        // 借记卡一档，二档
        utilSegmentFeeForCreditDebit(payUnionObject1, bankFee1.getDebitRate(), bankFee1.getDebitMin(), bankFee1.getDebitMax(), "1000.01", "999999.99");
        utilSegmentFeeForCreditDebit(payUnionObject1, bankFee2.getDebitRate(), bankFee2.getDebitMin(), bankFee2.getDebitMax(), "0", "1000.01");
        payUnionObject1.put("banktype", "9001002");  //银联扫码
        payUnionObject1.put("paymode", "23");
        updateScanBankObjList.add(payUnionObject1);

        JSONObject payUnionObject2 = initTempDetail(); // 贷记卡
        payUnionObject2.put("bankAccountType", "12");
        payUnionObject2.put("feeType", "3");
        utilSegmentFeeForCreditDebit(payUnionObject2, bankFee1.getCreditRate(), bankFee1.getCreditMin(), null, "1000.01", "999999.99");
        utilSegmentFeeForCreditDebit(payUnionObject2, bankFee2.getCreditRate(), bankFee2.getCreditMin(), null, "0", "1000.01");
        payUnionObject2.put("banktype", "9001002");  //银联扫码
        payUnionObject2.put("paymode", "23");
        updateScanBankObjList.add(payUnionObject2);

        //银联借贷记卡开通线上扫描支付费率模板
        JSONObject unionFront1 =JSONObject.parseObject(JSON.toJSONString(payUnionObject1, SerializerFeature.DisableCircularReferenceDetect));
        unionFront1.put("paymode", "07");
        unionFront1.put("banktype", "9001002");
        updateScanBankObjList.add(unionFront1);

        JSONObject unionFront2 = JSONObject.parseObject(JSON.toJSONString(payUnionObject2, SerializerFeature.DisableCircularReferenceDetect));
        unionFront2.put("paymode", "07");
        unionFront2.put("banktype", "9001002");
        updateScanBankObjList.add(unionFront2);

        return updateScanBankObjList;
    }


    private static JSONObject initTempDetail(){
        JSONObject detail = new JSONObject();
        detail.put("priority", "1"); // 类型：String 必有字段
        // 备注：优先级(整数，且范围为[0,9999]),优先级越小越优先
        detail.put("discount", 1); // 类型：Number 必有字段
        // 备注：收费折扣(1代表无折扣，0.8代表8折)，范围[0.01,1.00],仅支持小数点后2位
        detail.put("isLocal", "00");// 类型：String 必有字段 备注：是否是同城（00:不限;01:是;02:不是）
        return detail;
    }

    /**
     * 分段计费的封装类 （微信 支付宝）
     */
    private static void utilSegmentFee(Map<String,Object> feeMap, String percentStr, String lowValueStr, String maxAmt) {
        BigDecimal percent = new BigDecimal(percentStr).divide(new BigDecimal("100")).setScale(2, RoundingMode.FLOOR);
        BigDecimal percentRatio = percent.divide(new BigDecimal("100")).setScale(4, RoundingMode.FLOOR);
        BigDecimal lowValue = new BigDecimal(lowValueStr).divide(new BigDecimal("100")).setScale(2, RoundingMode.FLOOR);
        if (lowValue.compareTo(new BigDecimal("0")) == 0) {
            feeMap.put("feeType", "2");
            feeMap.put("fee", percent.doubleValue());
            feeMap.put("minAmount", 0d);
        } else {
            feeMap.put("feeType", "3");
            Map<String, Object> map1 = new HashMap<String, Object>();
            Map<String, Object> map2 = new HashMap<String, Object>();
            //固定计费
            BigDecimal map1End = (lowValue.divide(percentRatio, 2, RoundingMode.FLOOR)).add(new BigDecimal("0.01"));
            BigDecimal maxAmtBigDec = new BigDecimal(maxAmt).setScale(2, RoundingMode.FLOOR);
            if (map1End.compareTo(maxAmtBigDec) > 0) {
                feeMap.put("feeType", "1");
                feeMap.put("fee", lowValue.doubleValue());
                return;
            }
            map1.put("amountStart", 0d);
            map1.put("amountEnd", map1End.doubleValue());
            map1.put("fee", lowValue.doubleValue());
            // 费率计费
            map2.put("amountStart", map1End.doubleValue());
            map2.put("amountEnd", maxAmtBigDec.doubleValue());
            map2.put("feeRate", percent.doubleValue());
            List<Map<String, Object>> credit = new ArrayList<Map<String, Object>>(2);
            credit.add(map1);
            credit.add(map2);
            feeMap.put("feeSegmentRateList", credit);
        }
    }


    /**
     * 银联分段计费封装
     */
    private static void utilSegmentFeeForCreditDebit(Map<String, Object> feeMap, String percentStr, String lowValueStr, String highValueStr, String minAmt, String maxAmt) {
        List<Map<String, Object>> feeSegmentRateList = (List<Map<String, Object>>)feeMap.get("feeSegmentRateList");
        if (feeSegmentRateList == null) {
            feeSegmentRateList = new ArrayList<Map<String, Object>>();
        }
        BigDecimal maxAmtBigDec = new BigDecimal(maxAmt).setScale(2, RoundingMode.FLOOR);
        BigDecimal minAmtBigDec = new BigDecimal(minAmt).setScale(2, RoundingMode.FLOOR);
        BigDecimal rate = new BigDecimal(percentStr).divide(new BigDecimal("100"),2, RoundingMode.FLOOR);
        BigDecimal rateRadio = rate.divide(new BigDecimal("100"),4, RoundingMode.FLOOR);
        Map<String, Object> feeSementMap1 = new HashMap<String, Object>();
        //两者都没填的话走费率
        if (StringUtils.isBlank(lowValueStr) && StringUtils.isBlank(highValueStr)) {
            feeSementMap1.put("amountStart", minAmtBigDec.doubleValue());
            feeSementMap1.put("amountEnd", maxAmtBigDec.doubleValue());
            feeSementMap1.put("feeRate", rate.doubleValue());
            feeSegmentRateList.add(feeSementMap1);
        } else {
            Map<String, Object> feeSementMap3Temp = null;
            BigDecimal divideDecAdd = null;
            if (StringUtils.isNotBlank(lowValueStr)) {
                BigDecimal lowValueDec = new BigDecimal(lowValueStr).divide(new BigDecimal("100"), 2, RoundingMode.FLOOR);
                BigDecimal divideDec = lowValueDec.divide(rateRadio, 2, RoundingMode.FLOOR);
                if (divideDec.compareTo(minAmtBigDec) > 0) {
                    divideDecAdd = divideDec.add(new BigDecimal("0.01"));
                    Map<String, Object> feeSementMap2 = new HashMap<String, Object>();
                    feeSementMap2.put("amountStart", minAmtBigDec.doubleValue());
                    feeSementMap2.put("amountEnd", divideDecAdd.compareTo(maxAmtBigDec) < 0 ? divideDecAdd.doubleValue() : maxAmtBigDec.doubleValue());
                    feeSementMap2.put("fee", lowValueDec.doubleValue());
                    feeSegmentRateList.add(feeSementMap2);
                }
                if (divideDec.compareTo(maxAmtBigDec) < 0) {
                    feeSementMap3Temp = new HashMap<String, Object>();
                    feeSementMap3Temp.put("amountStart", divideDecAdd == null ? minAmtBigDec.doubleValue() : divideDecAdd.doubleValue());
                    feeSementMap3Temp.put("amountEnd", maxAmtBigDec.doubleValue());
                    feeSementMap3Temp.put("feeRate", rate.doubleValue());
                }
            }
            if (StringUtils.isNotBlank(highValueStr)) {
                BigDecimal highValueDec = new BigDecimal(highValueStr).divide(new BigDecimal("100"), 2, RoundingMode.FLOOR);
                if (highValueDec.compareTo(new BigDecimal("0")) != 0) {
                    BigDecimal divideHighDec = highValueDec.divide(rateRadio, 2, RoundingMode.FLOOR);
                    BigDecimal divideHighDecAdd =  divideHighDec.add(new BigDecimal(0.01));
                    if (divideHighDecAdd.compareTo(maxAmtBigDec) < 0 && divideHighDecAdd.compareTo(minAmtBigDec) > 0) {
                        if (feeSementMap3Temp != null) {
                            feeSementMap3Temp.put("amountEnd", divideHighDecAdd.doubleValue());
                        }
                        //说明只填写了封顶
                        if (StringUtils.isBlank(lowValueStr)) {
                            Map<String, Object> feeSementMap5 = new HashMap<String, Object>();
                            feeSementMap5.put("amountStart", minAmtBigDec.doubleValue());
                            feeSementMap5.put("amountEnd", divideHighDecAdd.doubleValue());
                            feeSementMap5.put("feeRate", rate.doubleValue());
                            feeSegmentRateList.add(feeSementMap5);
                        }
                        if (divideHighDecAdd.compareTo(maxAmtBigDec) < 0) {
                            Map<String, Object> feeSementMap4 = new HashMap<String, Object>();
                            feeSementMap4.put("amountStart", divideHighDecAdd.doubleValue());
                            feeSementMap4.put("amountEnd", maxAmtBigDec.doubleValue());
                            feeSementMap4.put("fee", highValueDec.doubleValue());
                            feeSegmentRateList.add(feeSementMap4);
                        }
                    }
                }
            }
            if (feeSementMap3Temp != null) {
                feeSegmentRateList.add(feeSementMap3Temp);
            }
        }
        feeMap.put("feeSegmentRateList", feeSegmentRateList);
    }


}
